/**
 *
 * @author DatecsSoft_Team
 */
package connectivity;

import com.datecs.fiscalprinter.SDK.FiscalExceptionEn;
import static com.datecs.fiscalprinter.SDK.FiscalExceptionEn.FPTR_CC_OTHER;
import com.datecs.fiscalprinter.SDK.FiscalPrinterV1;
import com.datecs.fiscalprinter.SDK.model.BGR.FDModelDetectorV1;
import com.datecs.fiscalprinter.SDK.model.BGR.FMP10_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.FP650_BGR;
import com.datecs.fiscalprinter.SDK.model.DatecsFiscalDevice;
import com.datecs.util.LoggerEx;
import com.fazecast.jSerialComm.SerialPort;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

public class SerialPrinterConnector implements PrinterConnection {

 

    final private LoggerEx logger = LoggerEx.getLogger(SerialPrinterConnector.class);
    private SerialPort serialPort = null;

   private static String modelName = "";

    public static String getModelName()
    {
        return modelName;
    }
 
    private int boudrate = 9600;
    private int timeOut = 1000;

    public SerialPrinterConnector(String modelName)
    {
        this.modelName = modelName;
    }

    public Object getConnection() throws Exception
    {
        return serialPort;
    }

    public void checkOpened() throws Exception
    {
        if (serialPort == null) {
            throw new Exception("Port is not opened");
        }
    }

    @Override
    public String getPortName() throws Exception
    {
        //Read Port Name
        return "";
    }

 

    @Override
    public DatecsFiscalDevice open(String portName) throws Exception
    {   DatecsFiscalDevice    connectedFD = new DatecsFiscalDevice(FPTR_CC_OTHER);
        try {               
            if (serialPort == null) {
                serialPort = SerialPort.getCommPort(portName);
                if (serialPort.openPort()) {
                    serialPort.setComPortParameters(boudrate, 8, 1, 0);
                    serialPort.setComPortTimeouts(SerialPort.TIMEOUT_NONBLOCKING, timeOut, 0);
                    serialPort.setFlowControl(SerialPort.FLOW_CONTROL_DISABLED);
                    InputStream in = serialPort.getInputStream();
                    OutputStream out = serialPort.getOutputStream();
                    //Autodetect model                      
                    FDModelDetectorV1 datecsBGRmodelV1 = new FDModelDetectorV1(
                            serialPort.getInputStream(),
                            serialPort.getOutputStream());
                    modelName = datecsBGRmodelV1.detectConnectedModel();                 
                    connectedFD.setConnectedModel(createPrinter(in, out));
                }else 
                    throw new Exception("Can't open port :" + portName);
                
                
            }
        } catch (RuntimeException | FiscalExceptionEn | IOException ex) {
            logger.error(ex);
            serialPort.closePort();
        }
        return connectedFD;
    }

    @Override
    public void close() throws IOException
    {
        if (isOpen()) {
            serialPort.getInputStream().close();
            serialPort.getOutputStream().close();
            logger.debug("\n\nClosing " + serialPort.getDescriptivePortName() + ": " + serialPort.closePort());
        }
    }

     private boolean isOpen()
    {
     if (serialPort==null) return false;
     return  serialPort.isOpen();
    }

    @Override
    public boolean isSearchByBaudRateEnabled()
    {
        return false;
    }

    @Override
    public void setBaudRate(int baudRate) throws Exception
    {
        if (this.boudrate != baudRate) {
            this.boudrate = baudRate;
        }

    }

    @Override
    public void setTimeout(int timeout) throws Exception
    {
        this.timeOut = timeout;
    }

    @Override
    public void setPortName(String portName) throws Exception
    {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    public InputStream getInputStream()
    {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    public OutputStream getOutputStream()
    {
        throw new UnsupportedOperationException("Not supported yet."); //To change body of generated methods, choose Tools | Templates.
    }

    private FiscalPrinterV1 createPrinter(InputStream i, OutputStream o) throws IOException, FiscalExceptionEn
    {
        //Use i90 name;
/*
         * 'DP-05',
         * 'DP-25',
         * 'DP-35',
         * 'DP-150'
         * 'WP-50',
         * 'WP-500'
         * 'FP-650'
         * 'FP-700'
         * 'FP-800'
         * 'FMP-350'
         * 'FP-650V'
         * 'FP-700V'
         * 'FP-800V'
         */

        switch (modelName) {
        case "DP-05":
            return null;// new DP05_ROU(i, o);
        case "DP-25":
            return null;// new DP25_ROU(i, o);
        case "DP-35":
            return null;// FP700_ROU(i, o);
        case "DP-150":
            return null;// null;// new DP150_ROU(i, o);
        case "WP-50":
            return null;// new WP50_ROU(i, o);
        case "WP-500":
            return null;// new WP500_ROU(i, o);
        case "FP-650":
             return new FP650_BGR(i, o);
        case "FP-700":
            return null;// new FP700_ROU(i, o);
        case "FP-800":
            return null;// new FP800_ROU(i, o);
        case "FMP-10":
         return new FMP10_BGR(i, o);

        default:
            return null;

        }
    }



}
