package com.datecs.demo;


import com.datecs.demo.connectivity.AbstractConnector;
import com.datecs.fiscalprinter.SDK.FiscalExceptionEn;
import com.datecs.fiscalprinter.SDK.FiscalPrinterV1;
import com.datecs.fiscalprinter.SDK.model.BGR.DP05_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.DP150_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.DP25_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.DP35_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.FDModelDetectorV1;
import com.datecs.fiscalprinter.SDK.model.BGR.FMP10_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.FP2000_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.FP550_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.FP650_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.FP800_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.SK1_21F_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.SK1_31F_BGR;
import com.datecs.fiscalprinter.SDK.model.BGR.WP50_BGR;
import com.datecs.fiscalprinter.SDK.model.DatecsFiscalDevice;
import com.datecs.fiscalprinter.SDK.model.UserLayerV1.cmdService;

import java.io.IOException;

import static com.datecs.fiscalprinter.SDK.FiscalExceptionEn.FPTR_CC_OTHER;

public class PrinterManager {

    private static String sConnectorType;
    private String modelVendorName = "";

    public String getModelVendorName() {
        return modelVendorName;
    }

    public DatecsFiscalDevice getFiscalDevice() {
        return fiscalDevice;
    }


    private DatecsFiscalDevice fiscalDevice;//communication with the connected device occurs only through this object

    private AbstractConnector mConnector;

    public static final PrinterManager instance;

    static {
        instance = new PrinterManager();
    }

    private PrinterManager() {
    }

    public static String getsConnectorType() {
        return sConnectorType;
    }

    public void init(AbstractConnector connector) throws IOException, FiscalExceptionEn {
        fiscalDevice = new DatecsFiscalDevice(FPTR_CC_OTHER);
        mConnector = connector;
        sConnectorType = connector.getConnectorType();

        //Protocol V1 -Identifies all 6 byte status devices !!!
        //Enables whether a status bytes of the device is a critical exception for user application !
        FiscalPrinterV1.setCriticalStatuses(initCriticalStatusPrinter());
        FiscalPrinterV1.setCriticalStatuses(initCriticalStatusPrinter());
        //Create an abstract device to determine the model of the connected device
        FDModelDetectorV1 datecsBGRmodelV1 = new FDModelDetectorV1(mConnector.getInputStream(), mConnector.getOutputStream());
        modelVendorName = datecsBGRmodelV1.detectConnectedModel();

        if (modelVendorName != "")
            switch (modelVendorName) {

                case "DP-05":
                    fiscalDevice.setConnectedModel(new DP05_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "DP-25":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new DP25_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "DP-35":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new DP35_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "WP-50":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new WP50_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "DP-150":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new DP150_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "FP-800":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new FP800_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "FP-650":
                    fiscalDevice.setConnectedModel(new FP650_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    //Low level command error messages are discarded as text and throw exceptions.
                    fiscalDevice.getConnectedPrinterV1().setThrowErrorMessage(true); //Turn Off/On  Error Messages in exceptions


                    //Turn ON Fiscal Device mode "Optional Text in Response":
                    new cmdService().setErrorMessageInResponse(true);
                    break;

                case "FMP-10":
                    fiscalDevice.setConnectedModel(new FMP10_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    //fiscalDevice.getConnectedPrinterV1().setThrowErrorMessage(false); //Turn Off  Error Messages
                    //fiscalDevice.getConnectedPrinterV1().setChkInputParams(false); //Turn Off Input Parameters Checking
                    break;
                case "FP-2000":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new FP2000_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "SK1-21F ":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new SK1_21F_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                case "SK1-31F":
                    //Not tested with device
                    fiscalDevice.setConnectedModel(new SK1_31F_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;
                case "FP-550":
                    fiscalDevice.setConnectedModel(new FP550_BGR(datecsBGRmodelV1.getTransportProtocol()));
                    break;

                default:
                    modelVendorName = "Unsupported model:" + modelVendorName;
                    break;
            }



    }

    /**
     * Enables whether a status is a critical exception
     *
     * @return
     */
    private boolean[][] initCriticalStatusPrinter() {

        boolean[][] myCriticalStatusSet = new boolean[6][8];
        myCriticalStatusSet[0][7] = false; //"For internal use  1."
        myCriticalStatusSet[0][6] = true;////"Cover is Open"
        myCriticalStatusSet[0][5] = false; // General error - this is OR of all errors marked with #.
        myCriticalStatusSet[0][4] = true;////"# Failure in printing mechanism"
        myCriticalStatusSet[0][3] = false;//"No client display connected"
        myCriticalStatusSet[0][2] = false;//"The real time clock is not synchronized"
        myCriticalStatusSet[0][1] = true;//"# Command code is invalid"
        myCriticalStatusSet[0][0] = true;//"# Syntax error"
        myCriticalStatusSet[1][7] = false;//"For internal use  1"
        myCriticalStatusSet[1][6] = true;//"The built-in tax terminal does not responding"
        myCriticalStatusSet[1][5] = false;//"An non fiscal receipt is opened to print a 90-degree text"
        myCriticalStatusSet[1][4] = false;//"Storno receipt is  Open"
        myCriticalStatusSet[1][3] = false;//"Low battery (Real time clock is RESET)"
        myCriticalStatusSet[1][2] = false;////"RAM Reset is done."
        myCriticalStatusSet[1][1] = true;//"# Command is not permitted"
        myCriticalStatusSet[1][0] = true;//"# Overflow during command execution"
        myCriticalStatusSet[2][7] = false;//"For internal use  1."
        myCriticalStatusSet[2][6] = false;//"EJ nearly full.(only few receipt allowed)"
        myCriticalStatusSet[2][5] = false;//"Nonfiscal receipt is openFiscalReceipt."
        myCriticalStatusSet[2][4] = false;//"EJ nearly full."
        myCriticalStatusSet[2][3] = false;//"Fiscal receipt is Open."
        myCriticalStatusSet[2][2] = false;//"EJ full." //TODO : For test only
        myCriticalStatusSet[2][1] = false;//"Near paper end."
        myCriticalStatusSet[2][0] = true;//"#End of paper."
        myCriticalStatusSet[3][7] = false; //"For internal use  1"
        myCriticalStatusSet[3][6] = false; //"Sw7"
        myCriticalStatusSet[3][5] = false; //"Sw6"
        myCriticalStatusSet[3][4] = false; //"Sw5"
        myCriticalStatusSet[3][3] = false; //"Sw4"
        myCriticalStatusSet[3][2] = false; //"Sw3"
        myCriticalStatusSet[3][1] = false; //"Sw2"
        myCriticalStatusSet[3][0] = false; //"Sw1"
        myCriticalStatusSet[4][7] = false;//"For internal use  1"
        myCriticalStatusSet[4][6] = true;//"The printer is overheated"
        myCriticalStatusSet[4][5] = false; //"OR of all errors marked with *  Bytes 4 - 5"
        myCriticalStatusSet[4][4] = true;//"* Fiscal memory is full."
        myCriticalStatusSet[4][3] = false; //"There is space for less then 50 reports in Fiscal memory."
        myCriticalStatusSet[4][2] = false; //"Serial number and number of FM is set"
        myCriticalStatusSet[4][1] = false;//"EIK on BULSTAT is set"
        myCriticalStatusSet[4][0] = true;//"* Error accessing data in the FM"
        myCriticalStatusSet[4][0] = true; //"* Error accessing data in the FM"
        myCriticalStatusSet[5][7] = false;//"For internal use  1."
        myCriticalStatusSet[5][6] = false;//"For internal use  0."
        myCriticalStatusSet[5][5] = true; //"Error reading  fiscal memory."
        myCriticalStatusSet[5][4] = false; //"VAT are not set at least once."
        myCriticalStatusSet[5][3] = false;//"Device is not fiscalized."
        myCriticalStatusSet[5][2] = true; //"The latest fiscal memory record failed."
        myCriticalStatusSet[5][1] = true;//"FM is not formated."
        myCriticalStatusSet[5][0] = true;//"The fiscal memory is set in READONLY mode (locked)"
        return myCriticalStatusSet;

    }

    /**
     * Enables whether a status is a critical exception
     *
     * @return
     */
    private boolean[][] initCriticalStatusECR() {

        boolean[][] myCriticalStatusSet = new boolean[6][8];
        myCriticalStatusSet[0][7] = false; //"For internal use  1."
        myCriticalStatusSet[0][6] = false;//"For internal use  0."
        myCriticalStatusSet[0][5] = true; // General error - this is OR of all errors marked with #.
        myCriticalStatusSet[0][4] = false;//""For internal use  0."
        myCriticalStatusSet[0][3] = false;//"No client display connected"
        myCriticalStatusSet[0][2] = false;//"The real time clock is not synchronized"
        myCriticalStatusSet[0][1] = true;//"# Command code is invalid"
        myCriticalStatusSet[0][0] = true;//"# Syntax error"
        myCriticalStatusSet[1][7] = false;//"For internal use  1"
        myCriticalStatusSet[1][6] = true;//"The built-in tax terminal does not responding"
        myCriticalStatusSet[1][5] = false;//"An non fiscal receipt is opened to print a 90-degree text"
        myCriticalStatusSet[1][4] = false;//"Storno receipt is  Open"
        myCriticalStatusSet[1][3] = false;//"For internal use  0."
        myCriticalStatusSet[1][2] = false;//"For internal use  0."
        myCriticalStatusSet[1][1] = true;//"# Command is not permitted"
        myCriticalStatusSet[1][0] = true;//"# Overflow during command execution"
        myCriticalStatusSet[2][7] = false;//"For internal use  1."
        myCriticalStatusSet[2][6] = false;//"EJ nearly full.(only few receipt allowed)"
        myCriticalStatusSet[2][5] = false;//"Nonfiscal receipt is openFiscalReceipt."
        myCriticalStatusSet[2][4] = false;//"EJ nearly full."
        myCriticalStatusSet[2][3] = false;//"Fiscal receipt is Open."
        myCriticalStatusSet[2][2] = false;//"EJ full." // TODO:
        myCriticalStatusSet[2][1] = false;//"Near paper end."
        myCriticalStatusSet[2][0] = true;//"#End of paper."
        myCriticalStatusSet[3][7] = false; //"For internal use  1"
        myCriticalStatusSet[3][6] = false; //"Sw7"
        myCriticalStatusSet[3][5] = false; //"Sw6"
        myCriticalStatusSet[3][4] = false; //"Sw5"
        myCriticalStatusSet[3][3] = false; //"Sw4"
        myCriticalStatusSet[3][2] = false; //"Sw3"
        myCriticalStatusSet[3][1] = false; //"Sw2"
        myCriticalStatusSet[3][0] = false; //"Sw1"
        myCriticalStatusSet[4][7] = false;//"For internal use  1"
        myCriticalStatusSet[4][6] = true;//"The printer is overheated"
        myCriticalStatusSet[4][5] = false; //"OR of all errors marked with *  Bytes 4 - 5"
        myCriticalStatusSet[4][4] = true;//"* Fiscal memory is full."
        myCriticalStatusSet[4][3] = false; //"There is space for less then 50 reports in Fiscal memory."
        myCriticalStatusSet[4][2] = false; //"Serial number and number of FM is set"
        myCriticalStatusSet[4][1] = false;//"EIK on BULSTAT is set"
        myCriticalStatusSet[4][0] = true;//"* Error accessing data in the FM"
        myCriticalStatusSet[4][0] = true; //"* Error accessing data in the FM"
        myCriticalStatusSet[5][7] = false;//"For internal use  1."
        myCriticalStatusSet[5][6] = false;//"For internal use  0."
        myCriticalStatusSet[5][5] = true; //"Error reading  fiscal memory."
        myCriticalStatusSet[5][4] = false; //"VAT are not set at least once."
        myCriticalStatusSet[5][3] = false;//"Device is not fiscalized."
        myCriticalStatusSet[5][2] = true; //"The latest fiscal memory record failed."
        myCriticalStatusSet[5][1] = true;//"FM is not formated."
        myCriticalStatusSet[5][0] = true;//"The fiscal memory is set in READONLY mode (locked)"
        return myCriticalStatusSet;

    }

    public void close() {
        if (mConnector != null) {
            try {
                mConnector.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }


}
