/**
 * This fragment provides a custom graphical user interface for document search in the device's
 * electronic journal (EJ).
 * Search can be done by document number by date-time
 * and the results can be filtered by document type.
 *
 * The following methods have been demonstrated as part of the Datecs Java SDK:
 *
 * @author Datecs Ltd. Software Department
 *
 */

package com.datecs.demo.ui.main;

import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.databinding.DataBindingUtil;
import android.os.Bundle;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.datecs.demo.EJ_frg_binding;
import com.datecs.demo.ui.main.tools.SetTime;
import com.datecs.demo.ui.main.tools.TextViewDatePicker;
import com.datecs.fiscalprinter.SDK.model.UserLayerV1.cmdEJournal;
import com.datecs.testApp.R;
import java.text.SimpleDateFormat;
import java.util.Date;

public class EJ_Fragment extends Fragment {
    private EJ_frg_binding binder;
    private ProgressDialog progress;

    @Override
    public View onCreateView(LayoutInflater inflater, ViewGroup container,
                             Bundle savedInstanceState) {
        binder = DataBindingUtil.inflate(inflater, R.layout.ej_fragment, container, false);
        try {
            SimpleDateFormat sdf = new SimpleDateFormat("dd-MM-yy");
            String currentDate = sdf.format(new Date());
            binder.edKLstartDate.setText(currentDate);
            binder.edKLtoDate.setText(currentDate);
            sdf = new SimpleDateFormat("HH:mm:ss");
            String currentTime = sdf.format(new Date());
            binder.edKLstartTime.setText(currentTime);
            binder.edKLtoTime.setText(currentTime);

        } catch (Exception e) {
            postToast(e.getMessage());
            e.printStackTrace();
        }
        return binder.getRoot();
    }

    @Override
    public void onViewCreated(final View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        final cmdEJournal myEJournal = new cmdEJournal();
        TextViewDatePicker datePickerFrom = new TextViewDatePicker(getContext(), binder.edKLstartDate);
        TextViewDatePicker datePickerTo = new TextViewDatePicker(getContext(), binder.edKLtoDate);
        new SetTime(getContext(), binder.edKLstartTime);
        new SetTime(getContext(), binder.edKLtoTime);


        binder.btnReadEJ.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                //if (!checkRange()) return;
                myEJournal.setUserBreak(false); //Pri povtoren print
                progress = new ProgressDialog(getContext());
                progress.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                progress.setIndeterminate(false);
                progress.setCancelable(false);
                progress.setButton(DialogInterface.BUTTON_NEGATIVE,
                        getString(R.string.cancel),
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                myEJournal.setUserBreak(true);
                                dialog.dismiss();
                            }
                        });
                final boolean isSearchByNumber = binder.chbKLrangeBynum.isChecked();
                final boolean isSearchInZreports = binder.chbKLrangeByZ.isChecked();
                if (isSearchByNumber) progress.setTitle(R.string.title_reading_documents_by_num);
                else progress.setTitle(R.string.title_reading_documents_by_dat);
                progress.show();
                final String[] toastText = {"0"};
                final String[] readedDocsList = new String[1];
                readedDocsList[0] = "";
                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        // do the thing that takes a long time
                        try {
                            if (isSearchByNumber) //Search by number
                            {

                                if (!isSearchInZreports)
                                    readedDocsList[0] = myEJournal.readEjDocumentsByNumbersRange(
                                            binder.edKLfromNo.getText().toString(),
                                            binder.edKLtoNo.getText().toString(),
                                            cmdEJournal.EjDocumentType.fromOrdinal((int) binder.spKLdocType.getSelectedItemId()));
                                else { //By Z report
                                    readedDocsList[0] = myEJournal.readEjDocumentsInZReport(
                                            binder.edKLfromNo.getText().toString(),
                                            binder.edKLtoNo.getText().toString(),
                                            binder.edToZrep.getText().toString(),
                                            cmdEJournal.EjDocumentType.fromOrdinal((int) binder.spKLdocType.getSelectedItemId()));
                                }

                            } else //..by DateTime
                            {
                                String formatStartDataTime = binder.edKLstartDate.getText().toString().replace("-", "") +
                                        binder.edKLstartTime.getText().toString().replace(":", "");
                                String formatEndDataTime = binder.edKLtoDate.getText().toString().replace("-", "") +
                                        binder.edKLtoTime.getText().toString().replace(":", "");


                                readedDocsList[0] = myEJournal.readEjDocumentsByDateRange(
                                        formatStartDataTime,
                                        formatEndDataTime,
                                        cmdEJournal.EjDocumentType.fromOrdinal((int) binder.spKLdocType.getSelectedItemId()));
                            }

                        } catch (Exception e) {
                            e.printStackTrace();
                            postToast(e.getMessage());
                            return;
                        } finally {
                            progress.dismiss();
                        }
                        getActivity().runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                binder.edEjTextMonitor.setText("");
                                binder.edEjTextMonitor.setMaxLines(readedDocsList[0].length());
                                binder.edEjTextMonitor.setText(readedDocsList[0]);

                                toastText[0] = getString(R.string.msg_doc_received) + String.valueOf(binder.edEjTextMonitor.getLineCount());
                                Snackbar.make(getView(), toastText[0], Snackbar.LENGTH_LONG).setAction("Action", null).show();

                            }
                        });
                    }
                }).start();

            }
        });

        binder.btnEJInfo.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                try {
              String infoEJ=myEJournal.readEjInfo();
                    binder.edEjTextMonitor.setText("");
                    binder.edEjTextMonitor.setMaxLines(infoEJ.length());
                    binder.edEjTextMonitor.setText(infoEJ);
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });
        binder.btnValidate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                try {
                    myEJournal.validateEj();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });
        binder.btnPrintDoc.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                myEJournal.setUserBreak(false); //Pri povtoren print
                progress = new ProgressDialog(getContext());
                progress.setProgressStyle(ProgressDialog.STYLE_SPINNER);
                progress.setIndeterminate(false);
                progress.setCancelable(false);
                progress.setButton(DialogInterface.BUTTON_NEGATIVE,
                        getString(R.string.cancel),
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                myEJournal.setUserBreak(true);
                                dialog.dismiss();
                            }
                        });

                final boolean isPrintByNumber = binder.chbKLrangeBynum.isChecked();
                final boolean isSearchInZreports = binder.chbKLrangeByZ.isChecked();
                if (isPrintByNumber) progress.setTitle(R.string.title_printing_documents_by_num);
                else progress.setTitle(R.string.title_printing_documents_by_dat);
                progress.show();
                final String[] toastText = {"0"};

                new Thread(new Runnable() {
                    @Override
                    public void run() {
                        // do the thing that takes a long time
                        try {

                            if (isPrintByNumber) //Search by number
                            {

                                if (!isSearchInZreports)
                                    myEJournal.printEjDocumentsByNumbersRange(
                                            binder.chbCondensedFont.isChecked(),
                                            binder.edKLfromNo.getText().toString(),
                                            binder.edKLtoNo.getText().toString(),
                                            cmdEJournal.EjDocumentType.fromOrdinal((int) binder.spKLdocType.getSelectedItemId()));
                                else {
                                    myEJournal.printEjDocumentsInZReport(
                                            binder.chbCondensedFont.isChecked(),
                                            binder.edKLfromNo.getText().toString(),
                                            binder.edKLtoNo.getText().toString(),
                                            binder.edToZrep.getText().toString(),
                                            cmdEJournal.EjDocumentType.fromOrdinal((int) binder.spKLdocType.getSelectedItemId()));

                                }

                            } else            //Print by Date
                            {
                                String formatStartDataTime = binder.edKLstartDate.getText().toString().replace("-", "") +
                                        binder.edKLstartTime.getText().toString().replace(":", "");
                                String formatEndDataTime = binder.edKLtoDate.getText().toString().replace("-", "") +
                                        binder.edKLtoTime.getText().toString().replace(":", "");

                                myEJournal.printEjDocumentsByDateRange(
                                        binder.chbCondensedFont.isChecked(),
                                        formatStartDataTime,
                                        formatEndDataTime,
                                        cmdEJournal.EjDocumentType.fromOrdinal((int) binder.spKLdocType.getSelectedItemId()));

                            }

                        } catch (Exception e) {
                            e.printStackTrace();
                            postToast(e.getMessage());
                            return;
                        } finally {
                            progress.dismiss();
                        }
                        getActivity().runOnUiThread(new Runnable() {
                            @Override
                            public void run() {
                                toastText[0] = getString(R.string.msg_doc_printed);
                                Snackbar.make(getView(), toastText[0], Snackbar.LENGTH_LONG).setAction("Action", null).show();
                            }
                        });
                    }
                }).start();

            }
        });

        binder.chbKLrangeByZ.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                boolean isZRange = binder.chbKLrangeByZ.isChecked();
                binder.tvToZrep.setEnabled(isZRange);
                binder.edToZrep.setEnabled(isZRange);

            }
        });

        binder.chbKLrangeBynum.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                boolean isNumberRange = binder.chbKLrangeBynum.isChecked();
                boolean isZRange = binder.chbKLrangeByZ.isChecked();
                //Setup control enable
                binder.chbKLrangeByZ.setEnabled(isNumberRange);
                binder.edKLstartDate.setEnabled(!isNumberRange);
                binder.edKLstartTime.setEnabled(!isNumberRange);
                binder.edKLtoDate.setEnabled(!isNumberRange);
                binder.edKLtoTime.setEnabled(!isNumberRange);
                binder.tvFromNo.setEnabled(isNumberRange);
                binder.tvToNo.setEnabled(isNumberRange);
                binder.edKLfromNo.setEnabled(isNumberRange);
                binder.edKLtoNo.setEnabled(isNumberRange);
                binder.tvToZrep.setEnabled(isNumberRange && isZRange);
                binder.edToZrep.setEnabled(isNumberRange && isZRange);

            }
        });

    }


    private boolean checkRange() {
        if (!binder.edKLfromNo.getText().toString().trim().matches("^\\d+$")) {
            binder.edKLfromNo.requestFocus();
            return false;
        } else if (!binder.edKLtoNo.getText().toString().trim().matches("^\\d+$")) {
            binder.edKLtoNo.requestFocus();
            return false;
        } else if (!binder.edToZrep.getText().toString().trim().matches("^\\d+$")) {
            binder.edToZrep.requestFocus();
            return false;
        } else return true;

    }


    private void postToast(final String text) {
        this.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getActivity(), text, Toast.LENGTH_LONG).show();
            }
        });
    }


}

