
package com.datecs.demo.ui.main;

import android.databinding.DataBindingUtil;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.design.widget.Snackbar;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;

import com.datecs.demo.MenuFrg_binder;
import com.datecs.fiscalprinter.SDK.BuildInfo;
import com.datecs.fiscalprinter.SDK.model.UserLayerV1.cmdInfo;
import com.datecs.fiscalprinter.SDK.model.UserLayerV1.cmdReceipt;
import com.datecs.fiscalprinter.SDK.model.UserLayerV1.cmdReport;
import com.datecs.fiscalprinter.SDK.model.UserLayerV1.cmdService;
import com.datecs.testApp.R;

public class MenuFragment extends Fragment {

    private MenuFrg_binder binder;
    private cmdReceipt.FiscalReceipt fiscalReceipt;
    private cmdReceipt.NonFiscalReceipt noFiscalReceipt;


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {

        binder = DataBindingUtil.inflate(inflater, R.layout.menu_fragment, container, false);
        fiscalReceipt = new cmdReceipt.FiscalReceipt();
        noFiscalReceipt = new cmdReceipt.NonFiscalReceipt();
        binder.tvSDKBuild.setText("Datecs JSDK Build: " + BuildInfo.VERSION);
        return binder.getRoot();
    }

    @Override
    public void onViewCreated(final View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);

        binder.btnSetDate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new DateTimeFragment(), "date_time_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });


        binder.btnHeaderFooter.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new HeaderFooterFragment(), "header_footer_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });

        binder.btnPrintConfig.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new PrintConfigFragment(), "print_config_frg");
                    ft.addToBackStack(null);
                    ft.commit();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnDeviceStatus.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new DeviceStatusFragment(), "device_status_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });

        binder.btnZReport.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    Double[] total = new Double[10];
                    cmdReport cmd = new cmdReport();
                    int num = cmd.PrintZreport(total);
                    postToast(String.valueOf(num));
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnXReport.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    Double[] total = new Double[10];
                    cmdReport cmd = new cmdReport();
                    int num = cmd.PrintXreport(total);
                    postToast(String.valueOf(num));
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnExSaleTest.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                try {
                    if (!fiscalReceipt.isOpen()) {
                        //ОТВАРЯНЕ НА ФИСКАЛЕН (КЛИЕНТСКИ) БОН
                        String operatorCode = "1";// Номер на оператор /1 до 16/
                        String operatorPassword = "0000"; //Операторска парола /4 до 8 цифри/
                        String salePoint = "1"; //Номер на касово място /цяло число от 1 до 99999/

                        /**
                         *
                         * UNP Unique sales number format:
                         * - serial number of FM "
                         * - four digits or Latin characters (operator code)
                         * - sequential sales number (seven digits with leading zeros)
                         * example: DT000600-0001-0001000
                         *
                         *   Before the first sale, the UNP must be set at least once
                         *if then omitted the parameter FU will increment with the
                         *number of the sale automatically.
                         */
                        String unp = new cmdInfo().GetDeviceSerialNumber() + "-" +
                                String.format("%04d", Integer.parseInt(operatorCode)) + "-" + // Pad left with trailing zero
                                String.format("%07d", 1 + Integer.valueOf(fiscalReceipt.getLastDocNumber()));  //Next Document number pad left with trailing zero


                        fiscalReceipt.openFiscalReceipt(operatorCode, operatorPassword, salePoint, unp);
                        //РЕГИСТРИРАНЕ (ПРОДАЖБА) НА СТОКА
                        cmdReceipt.FiscalReceipt.FiscalSale testSale = new cmdReceipt.FiscalReceipt.FiscalSale();

                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash);
                        fiscalReceipt.printFreeText("Тест продажба без параметри!", true, true, true, cmdReceipt.FiscalReceipt.FreeFiscalTextType.type32dpiA);
                        testSale.add(
                                "",
                                "",
                                "Б", //А, Б, В...
                                "0.01", "", "",
                                cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.noCorecction,
                                "");

                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        testSale.add(
                                "Демонстрация",
                                "продажба с количество",
                                "B", //A,B,C,...
                                "0.01", "1", "Кг",
                                cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.noCorecction,
                                "");

                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash);
                        testSale.add(
                                "Демонстрация",
                                "Процентна отстъпака",
                                "B", //A,B,C,...
                                "0.11", "10", "Кг",
                                cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.discountPercentage,
                                "-10"); // Use sign !
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        testSale.add(
                                "Демонстрация",
                                "Отстъпака по стойност",
                                "B", //A,B,C,...
                                "0.11", "10", "Литри",
                                cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.discountSum,
                                ".5");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.equal);
                        testSale.add(
                                "Демонстрация",
                                "Надбавка в процент",
                                "B", //A,B,C,...
                                "0.1", "10", "Литри",
                                cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.surchargePercentage,
                                "1.5");

                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.equal);
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash);
                        testSale.add(
                                "",
                                "VOID !!!",
                                "B", //A,B,C,...
                                "-0.1",
                                "",
                                "",
                                cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.noCorecction,
                                "");

                        //SUBTOTAL
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash);
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.equal);
                        testSale.subtotal(true, true, cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.noCorecction, "");
                        fiscalReceipt.printFreeText("Отстъпка в междинната сума", true, true, true, cmdReceipt.FiscalReceipt.FreeFiscalTextType.type32dpiB);
                        testSale.subtotal(true, true, cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.discountSum, "2.52");

                        //TOTAL
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash);
                        cmdReceipt.TotalResult totalResult = testSale.saleTotal(
                                "Тотал:",
                                "",
                                cmdReceipt.FiscalReceipt.FiscalSale.PaidMode.fromOrdinal(0).getId(),
                                "0.01");

                        fiscalReceipt.printFreeText("Не се дължи плащане!", true, true, true, cmdReceipt.FiscalReceipt.FreeFiscalTextType.type32dpiA);
                        //CLOSE
                        fiscalReceipt.closeFiscalReceipt();
                    } else fiscalReceipt.cancel();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnSaleTest.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                try {
                    if (!fiscalReceipt.isOpen()) {
                        //ОТВАРЯНЕ НА ФИСКАЛЕН (КЛИЕНТСКИ) БОН
                        String operatorCode = "1";// Номер на оператор /1 до 16/
                        String operatorPassword = "0000"; //Операторска парола /4 до 8 цифри/
                        String salePoint = "1"; //Номер на касово място /цяло число от 1 до 99999/
                        fiscalReceipt.openFiscalReceipt(operatorCode, operatorPassword, salePoint, "");
                        //РЕГИСТРИРАНЕ (ПРОДАЖБА) НА СТОКА
                        cmdReceipt.FiscalReceipt.FiscalSale testSale =
                                new cmdReceipt.FiscalReceipt.FiscalSale();

                        testSale.add(
                                "Демонстрация",
                                "Тестова продажба",
                                "Б", //А,Б...З
                                "0.01", "1", "Кг",
                                cmdReceipt.FiscalReceipt.FiscalSale.CorrectionType.noCorecction,
                                "");
                        cmdReceipt.TotalResult totalResult = testSale.saleTotal(
                                "Тотал:",
                                "",
                                cmdReceipt.FiscalReceipt.FiscalSale.PaidMode.fromOrdinal(0).getId(),
                                "");
                        fiscalReceipt.closeFiscalReceipt();
                    } else fiscalReceipt.cancel();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnCancelAll.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    if (fiscalReceipt.isOpen()) {
                        fiscalReceipt.cancel();
                    }
                    if (noFiscalReceipt.isOpen()) {
                        noFiscalReceipt.closeNonFiscalReceipt();
                    }

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnCustomCommand.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    DialogCustomCommand d = new DialogCustomCommand(getActivity());
                    d.show();
                    d.setTitle("Execute Command");
                    d.show();
                    DisplayMetrics displaymetrics = new DisplayMetrics();
                    getActivity().getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
                    int width = (int) ((int) displaymetrics.widthPixels * 0.9);
                    int height = (int) ((int) displaymetrics.heightPixels * 0.9);
                    d.getWindow().setLayout(width, height);

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnStorno.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                String operatorCode = "1";// Номер на оператор /1 до 16/
                String operatorPassword = "0000"; //Операторска парола /4 до 8 цифри/
                String salePoint = "1"; //Номер на касово място /цяло число от 1 до 99999/
                try {

                    /**
                     *
                     * UNP Unique sales number format:
                     * - serial number of FM "
                     * - four digits or Latin characters (operator code)
                     * - sequential sales number (seven digits with leading zeros)
                     * example: DT000600-0001-0001000
                     *
                     *   Before the first storno, the UNP must be set at least once
                     *   if then omitted the parameter FU will increment with the
                     *   number of the sale automatically.
                     *
                     */
                    String unp = new cmdInfo().GetDeviceSerialNumber() + "-" +
                            String.format("%04d", Integer.parseInt(operatorCode)) + "-" + // Pad left with trailing zero
                            String.format("%07d", 1 + Integer.valueOf(fiscalReceipt.getLastDocNumber()));  //Next Document number pad left with trailing zero


                    new cmdReceipt.FiscalReceipt.Storno(
                            operatorCode,
                            operatorPassword,
                            salePoint,
                            unp, //
                            cmdReceipt.FiscalReceipt.Storno.StornoType.OperatorMistake.getId(),
                            fiscalReceipt.getLastDocNumber(),
                            "Продажбата е с тестова цел.").printStornoReceipt();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnNonFiscalReceipt.setOnClickListener(new View.OnClickListener() {


            @Override
            public void onClick(View v) {
                try {
                    noFiscalReceipt.open();
                    noFiscalReceipt.printNonFiscalText("Благодарим Ви!");
                    noFiscalReceipt.closeNonFiscalReceipt();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnInfoEJ.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new EJ_Fragment(), "ej_frg");
                    ft.addToBackStack(null);
                    ft.commit();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnItems.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new ItemsFragment(), "items_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });


        binder.btnVatsDialog.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    DialogVAT d = new DialogVAT(getActivity());
                    d.show();
                    d.setTitle("Changing VAT rates");
                    d.show();
                    DisplayMetrics displaymetrics = new DisplayMetrics();
                    getActivity().getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
                    int width = (int) ((int) displaymetrics.widthPixels * 0.9);
                    int height = (int) ((int) displaymetrics.heightPixels * 0.9);
                    d.getWindow().setLayout(width, height);

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


    }

    private void postToast(final String message) {
        this.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getActivity(), message, Toast.LENGTH_LONG).show();
            }
        });
    }

}
