/*
 * @author Datecs Ltd. Software Department
 */

package com.datecs.demo;

import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.IntentFilter;
import android.hardware.usb.UsbDevice;
import android.hardware.usb.UsbManager;
import android.os.Bundle;
import android.os.PersistableBundle;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentManager;
import android.support.v4.app.FragmentTransaction;
import android.support.v7.app.AlertDialog;
import android.support.v7.app.AppCompatActivity;
import android.view.MenuItem;
import android.widget.TextView;

import com.datecs.demo.ui.main.MenuFragment;
import com.datecs.fiscalprinter.SDK.FiscalExceptionEn;
import com.datecs.fiscalprinter.SDK.model.DatecsFiscalDevice;
import com.datecs.testApp.R;

import java.io.IOException;

public class MainActivity extends AppCompatActivity {
    public static final int DATECS_USB_VID = 65520;
    public static final int FTDI_USB_VID = 1027;
    private static final String ACTION_USB_DETACHED = "android.hardware.usb.action.USB_DEVICE_DETACHED";
    public static int CONNECT_DEVICE = 1984;
    private TextView mTitle;
    public static DatecsFiscalDevice myFiscalDevice = null;
    private final BroadcastReceiver mUsbReceiver = new BroadcastReceiver() {
        public void onReceive(Context context, Intent intent) {
            String action = intent.getAction();
            if (UsbManager.ACTION_USB_DEVICE_DETACHED.equals(action)) {
                synchronized (this) {
                    UsbDevice device = intent.getParcelableExtra(UsbManager.EXTRA_DEVICE);
                    if ((device.getVendorId() == DATECS_USB_VID) ||
                            (device.getVendorId() == FTDI_USB_VID)) startConnect();
                }
            }
        }
    };

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        IntentFilter filter = new IntentFilter(ACTION_USB_DETACHED);
        registerReceiver(mUsbReceiver, filter);
        setContentView(R.layout.main_activity);
        // add back arrow to toolbar
        if (getSupportActionBar() != null) {
            getSupportActionBar().setDisplayHomeAsUpEnabled(true);
            getSupportActionBar().setDisplayShowHomeEnabled(true);
        }

        if (savedInstanceState == null) {
            myFiscalDevice = null;
            startConnect();
        }
    }


    private void startConnect() {
        Context context = getApplicationContext();
        Intent intent = new Intent(context, ConnectorActivity.class);
        startActivityForResult(intent, CONNECT_DEVICE);
    }

    @Override
    public void onActivityResult(int requestCode, int resultCode, Intent data) {
        super.onActivityResult(requestCode, resultCode, data);

        if (requestCode == CONNECT_DEVICE) {
            if (resultCode == MainActivity.RESULT_CANCELED) if (myFiscalDevice == null) finish();
            if (data == null) finish();

            if ((resultCode == MainActivity.RESULT_OK) && (requestCode == CONNECT_DEVICE)) {
                myFiscalDevice = PrinterManager.instance.getFiscalDevice();
                String s1 = PrinterManager.instance.getModelVendorName();
                String s2 = PrinterManager.getsConnectorType();
                getSupportActionBar().setTitle(getTitle() + "    " + s1 + ":" + s2);

                if (myFiscalDevice != null) ShowMenu();
                else finish();
            }
        }

    }

    private void ShowMenu() {
        FragmentTransaction ft = getSupportFragmentManager().beginTransaction();
        ft.replace(R.id.container, new MenuFragment(), "menu_frg");
        ft.addToBackStack(null);
        ft.commit();
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        if (item.getItemId() == android.R.id.home) {
//            finish();
            if (getSupportFragmentManager().getBackStackEntryCount() == 1) startConnect();
            else if (getSupportFragmentManager().getBackStackEntryCount() == 0) showExitDialog();
            else super.onBackPressed();
        }
        return super.onOptionsItemSelected(item);
    }

    private void showExitDialog() {
        AlertDialog.Builder dialog = new AlertDialog.Builder(this);
        dialog.setMessage(R.string.msg_q_exit);
        dialog.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
            @Override
            public void onClick(DialogInterface dialog, int which) {
                if (myFiscalDevice != null) {
                    try {
                        myFiscalDevice.close();
                    } catch (IOException e) {
                        e.printStackTrace();
                    } catch (FiscalExceptionEn fiscalExceptionEn) {
                        fiscalExceptionEn.printStackTrace();
                    }
                }
                finish();
            }
        });
        dialog.setNegativeButton(R.string.no, null);
        dialog.show();
    }

    @Override
    public void onBackPressed() {
        if (!BackStackFragment.handleBackPressed(getSupportFragmentManager())) {
            showExitDialog();
        }
    }


    public abstract static class BackStackFragment extends Fragment {
        public static boolean handleBackPressed(FragmentManager fm) {
            if (fm.getFragments() != null) {
                for (Fragment frag : fm.getFragments()) {
                    if (frag != null && frag.isVisible() && frag instanceof BackStackFragment) {
                        if (((BackStackFragment) frag).onBackPressed()) {
                            return true;
                        }
                    }
                }
            }
            return false;
        }

        protected boolean onBackPressed() {
            FragmentManager fm = getChildFragmentManager();
            if (handleBackPressed(fm)) {
                return true;
            } else if (getUserVisibleHint() && fm.getBackStackEntryCount() > 0) {
                fm.popBackStack();
                return true;
            }
            return false;
        }
    }



}
