/**
 *
 * This fragment provides a custom graphical user interface to edit and add items (articles) in the
 * device built-in database of articles items.
 *
 * @author Datecs Ltd. Software Department
 */
package com.datecs.demo.ui.main;

import android.content.DialogInterface;
import android.databinding.DataBindingUtil;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v7.app.AlertDialog;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.Toast;
import com.datecs.demo.ItemsFrg_bind;
import com.datecs.demo.MainActivity;
import com.datecs.fiscalprinter.SDK.model.UserLayerV2.cmdItems;
import com.datecs.testApp.R;

import org.apache.commons.lang3.math.NumberUtils;

public class ItemsFragment extends Fragment {
    private ItemsFrg_bind binder;

    public static enum TaxGroup {
        A("A"),
        B("B"),
        C("C"),
        D("D"),
        E("E"),
        F("F"),
        G("G"),
        H("H");
        private final String id;
        private static TaxGroup[] allValues = values();

        private TaxGroup(String id) {
            this.id = id;
        }

        public String getId() {
            return String.valueOf(this.id);
        }

        public static TaxGroup fromOrdinal(int n) {
            return allValues[n];
        }
    }


    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {


        binder = DataBindingUtil.inflate(inflater, R.layout.items_fragment, container, false);

        Integer stockGrCnt = Integer.valueOf(MainActivity.myFiscalDevice.getConnectedModelV2().getMaxStockGroup());
        //Init Stock Groups Spinner 1-99
        String[] items = new String[stockGrCnt];
        for (int i = 0; i < stockGrCnt; i++) //0..99
            items[i] = String.valueOf(i + 1); //1...stockGrCnt
        ArrayAdapter<String> adapter = new ArrayAdapter<String>(getContext(), android.R.layout.simple_spinner_dropdown_item, items);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        binder.spStockGr.setAdapter(adapter);

        Integer departmentsCnt = Integer.valueOf(MainActivity.myFiscalDevice.getConnectedModelV2().getMaxDepartment());
        //Init Department Spinner 0-99
        items = new String[departmentsCnt];
        for (int i = 0; i < departmentsCnt; i++)//0..99
            items[i] = String.valueOf(i); //0...departmentsCnt
        adapter = new ArrayAdapter<String>(getContext(), android.R.layout.simple_spinner_dropdown_item, items);
        adapter.setDropDownViewResource(android.R.layout.simple_spinner_dropdown_item);
        binder.spItemDept.setAdapter(adapter);
        return binder.getRoot();
    }

    @Override
    public void onViewCreated(final View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);
        final cmdItems myItems = new cmdItems();
        binder.btnReadItem.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                cmdItems.ItemModel item;
                Integer PLU = NumberUtils.toInt(binder.edItemID.getText().toString(), 1);
                try {
                    item = myItems.ReadItem(PLU);
                    if (item.getPLU() != "") {
                        binder.edItemName.setText(item.getName());
                        binder.spItemVAT.setSelection(Integer.valueOf(item.getTaxGr()) - 1);
                        binder.edItemPrice.setText(item.getPrice());
                        binder.edItemTurnover.setText(item.getTurnover());
                        binder.edStockQTY.setText(item.getQuantity());
                        binder.edItemSoldQTY.setText(item.getSoldQty());
                    } else postToast(getString(R.string.msg_not_found));
                } catch (Exception e) {
                    e.printStackTrace();
                    postToast(e.getMessage());
                }
            }

        });
        binder.btnItemDelete.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                AlertDialog.Builder dialog = new AlertDialog.Builder(getContext());
                dialog.setMessage(R.string.msg_delete_all);
                dialog.setPositiveButton(R.string.yes, new DialogInterface.OnClickListener() {
                    @Override
                    public void onClick(DialogInterface dialog, int which) {
                        try {
                            if (binder.chcxDelAll.isChecked())
                                myItems.DelAllItems();
                            else
                                myItems.DelItemsInRange(Integer.valueOf(binder.edItemID.getText().toString()), 0);

                        } catch (Exception e) {
                            postToast(e.getMessage());
                            e.printStackTrace();
                        }
                    }
                });
                dialog.setNegativeButton(R.string.no, null);
                dialog.show();

            }

        });

        binder.btnItemSaveToDevice.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                final cmdItems.ItemModel myItemModel = new cmdItems.ItemModel(
                        binder.edItemID.getText().toString(),
                        TaxGroup.fromOrdinal((int) binder.spItemVAT.getSelectedItemId()).getId(),           //A,B..H
                        binder.spItemDept.getSelectedItem().toString(),
                        binder.spStockGr.getSelectedItem().toString(),
                        String.valueOf(binder.spItemPriceType.getSelectedItemId()),
                        binder.edItemPrice.getText().toString(),
                        "A",
                        binder.edStockQTY.getText().toString(),
                        binder.edBarcode1.getText().toString(),
                        binder.edBarcode2.getText().toString(),
                        binder.edBarcode3.getText().toString(),
                        binder.edBarcode4.getText().toString(),
                        binder.edItemName.getText().toString(), "",
                        "", cmdItems.ItemModel.ItemState.ITEM_SAVED
                );

                try {
                    myItems.SetItem(myItemModel);
                    binder.edItemID.setText(
                            String.valueOf(Integer.valueOf(binder.edItemID.getText().toString()) + 1
                            )
                    );
                    binder.edItemName.setText("Item " + binder.edItemID.getText().toString());

                } catch (Exception e) {
                    e.printStackTrace();
                    postToast(e.getMessage());
                }
            }


        });
        binder.btnFindItem.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Integer PLU = NumberUtils.toInt(binder.edItemID.getText().toString(), 1); // Start from PLU
                try {
                    if (binder.chkFindFromTop.isChecked())
                        binder.edItemID.setText(String.valueOf(myItems.GetLastNotProgrammed(PLU)));
                    else
                        binder.edItemID.setText(String.valueOf(myItems.GetFirstNotProgrammed(PLU)));

                    binder.edItemName.setText("New item " + binder.edItemID.getText().toString());
                    //Clear all
                    binder.spItemVAT.setSelection(0);
                    binder.spStockGr.setSelection(0);
                    binder.chkWithTurnover.setChecked(false);
                    binder.edStockQTY.setText("");
                    binder.edItemSoldQTY.setText("");
                    binder.edItemTurnover.setText("");
                    binder.edItemPrice.setText("0.01");
                    binder.edItemPrice.requestFocus();

                } catch (Exception e) {
                    e.printStackTrace();
                    postToast(e.getMessage());
                }
            }

        });

        binder.btnReadFirst.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                cmdItems.ItemModel item;
                Integer PLU = NumberUtils.toInt(binder.edItemID.getText().toString(), 1);
                try {
                    if (binder.chkWithTurnover.isChecked())
                        item = myItems.GetFirstFoundWithSales(PLU);
                    else
                        item = myItems.GetFirstFoundProgrammed(PLU);
                    if (item.getPLU() != "") {
                        binder.edItemID.setText(item.getPLU());
                        binder.edItemName.setText(item.getName());
                        binder.edItemPrice.setText(item.getPrice());
                        binder.edItemTurnover.setText(item.getTurnover());
                        binder.edStockQTY.setText(item.getQuantity());
                        binder.edItemSoldQTY.setText(item.getSoldQty());
                    } else postToast(getString(R.string.msg_not_found));
                } catch (Exception e) {
                    e.printStackTrace();
                    postToast(e.getMessage());
                }
            }

        });

        binder.btnReadNext.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                cmdItems.ItemModel item;
                try {
                    if (binder.chkWithTurnover.isChecked()) item = myItems.GetNextFoundWithSales();
                    else item = myItems.GetNextProgrammed();

                    if (item.getPLU().equals("")) postToast(getString(R.string.msg_not_found));
                    else {
                        binder.edItemID.setText(item.getPLU());
                        binder.edItemName.setText(item.getName());
                        binder.edItemPrice.setText(item.getPrice());
                        binder.edItemTurnover.setText(item.getTurnover());
                        binder.edStockQTY.setText(item.getQuantity());
                        binder.edItemSoldQTY.setText(item.getSoldQty());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    postToast(e.getMessage());
                }
            }

        });

        binder.btnReadLast.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                cmdItems.ItemModel item;
                Integer PLU = NumberUtils.toInt(binder.edItemID.getText().toString(), 0);
                try {
                    if (binder.chkWithTurnover.isChecked())
                        item = myItems.GetLastFoundWithSales(PLU);
                    else
                        item = myItems.GetLastFoundProgrammed(PLU);
                    if (item.getPLU().equals("")) postToast(getString(R.string.msg_not_found));
                    else {

                        binder.edItemID.setText(item.getPLU());
                        binder.edItemName.setText(item.getName());
                        binder.edItemPrice.setText(item.getPrice());
                        binder.edItemTurnover.setText(item.getTurnover());
                        binder.edStockQTY.setText(item.getQuantity());
                        binder.edItemSoldQTY.setText(item.getSoldQty());
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                    postToast(e.getMessage());
                }
            }

        });
        binder.btnSaveQTY.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                cmdItems.ItemModel item;
                Integer PLU = NumberUtils.toInt(binder.edItemID.getText().toString(), 1);
                try {
                    myItems.SetItemQuantity(Integer.valueOf(binder.edItemID.getText().toString()),
                            Double.valueOf(binder.edStockQTY.getText().toString()));
                    item = myItems.ReadItem(PLU);
                    binder.edStockQTY.setText(item.getQuantity());
                } catch (Exception e) {
                    e.printStackTrace();
                    postToast(e.getMessage());
                }
            }

        });


    }


    private void postToast(final String message) {
        this.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getActivity(), message, Toast.LENGTH_LONG).show();
            }
        });
    }

}
