/*
 * @author Datecs Ltd. Software Department
 */

package com.datecs.demo.ui.main;

import android.databinding.DataBindingUtil;
import android.os.Bundle;
import android.support.annotation.NonNull;
import android.support.annotation.Nullable;
import android.support.v4.app.Fragment;
import android.support.v4.app.FragmentTransaction;
import android.util.DisplayMetrics;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Toast;
import com.datecs.fiscalprinter.SDK.BuildInfo;
import com.datecs.fiscalprinter.SDK.model.UserLayerV2.cmdConfig;
import com.datecs.fiscalprinter.SDK.model.UserLayerV2.cmdInfo;
import com.datecs.fiscalprinter.SDK.model.UserLayerV2.cmdReceipt;
import com.datecs.fiscalprinter.SDK.model.UserLayerV2.cmdReport;
import com.datecs.testApp.R;
import com.datecs.testApp.ui.MenuFrg_binder;

public class MenuFragment extends Fragment {

    private MenuFrg_binder binder;
    private cmdReceipt.FiscalReceipt fiscalReceipt;
    private cmdReceipt.NonFiscalReceipt noFiscalReceipt;

    private Integer DocNum=null; // Number of the last document need for storno operation
    private String DateTime=null; //Date and time of the last issued document  need for storno operation

    @Nullable
    @Override
    public View onCreateView(@NonNull LayoutInflater inflater, @Nullable ViewGroup container,
                             @Nullable Bundle savedInstanceState) {

        binder = DataBindingUtil.inflate(inflater, R.layout.menu_fragment, container, false);
        //Create fiscal receipt  instance
        fiscalReceipt = new cmdReceipt.FiscalReceipt();
        noFiscalReceipt = new cmdReceipt.NonFiscalReceipt();
        //Create  non fiscal receipt instance
        binder.tvSDKBuild.setText("Datecs JSDK Build: " + BuildInfo.VERSION);

        return binder.getRoot();
    }

    @Override
    public void onViewCreated(final View view, @Nullable Bundle savedInstanceState) {
        super.onViewCreated(view, savedInstanceState);


        binder.btnSetDate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new DateTimeFragment(), "date_time_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });


        binder.btnHeaderFooter.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new HeaderFooterFragment(), "header_footer_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });


        binder.btnDeviceStatus.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new DeviceStatusFragment(), "device_status_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });


        binder.btnZReport.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    Double[] total = new Double[10];
                    cmdReport cmd = new cmdReport();
                    int num = cmd.PrintZreport(total);
                    postToast(String.valueOf(num));
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnXReport.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    Double[] total = new Double[10];
                    cmdReport cmd = new cmdReport();
                    int num = cmd.PrintXreport(total);
                    postToast(String.valueOf(num));
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });
/**
 *
 * Here are a set of Java SDK methods for creating and opening a fiscal receipt,
 * creating a sales and add items in the receipt.
 * Payment of purchases, and closing the receipt.
 *
 */

        binder.btnSaleTest.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {

                String operatorCode = "1";// Operator number from 1...30;
                String operatorPassword = "0000"; //Operator password, ascii string of digits. Lenght from 1...8;
                String salePoint = "1"; //Number of point of sale from 1...99999;
                try {
                    if (!fiscalReceipt.isOpen()) {

                        //Open Fiscal bon in current receipt
                        DocNum = fiscalReceipt.openFiscalReceipt(
                                operatorCode,
                                operatorPassword,
                                salePoint,
                                false);
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash);

                        //Registration of item for sale
                        cmdReceipt.FiscalReceipt.FiscalSale testSale =
                                new cmdReceipt.FiscalReceipt.FiscalSale(
                                        "Бонбон",
                                        "1",
                                        "0.01",
                                        "",
                                        cmdReceipt.FiscalSale.DiscountType.noDiscount,
                                        "",
                                        "").add();

                        //TOTAL
                        testSale.saleTotal(
                                cmdReceipt.FiscalSale.PaymentType.cash,
                                ""
                        );
                        DocNum = fiscalReceipt.closeFiscalReceipt();
                        // So we read device data time to store for future Storno operation.
                        DateTime = new cmdConfig.DateTime().getDateTime();

                    } else fiscalReceipt.cancel();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

/**
 *
 * Here are a set of Java SDK methods for creating and opening a fiscal receipt,
 * creating a sales and add items in the receipt,
 * void of sales items, printing of text, and separating lines,
 * Payment of purchases, and closing the receipt.
 *
 */
        binder.btnSaleTestEx.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {

                String operatorCode = "1";// Operator number from 1...30;
                String operatorPassword = "0000"; //Operator password, ascii string of digits. Lenght from 1...8;
                String salePoint = "1"; //Number of point of sale from 1...99999;
                String NSale; //Unique sale number (21 chars "LLDDDDDD-CCCC-DDDDDDD"


                try {
                    if (!fiscalReceipt.isOpen()) {
                        //Creating the unique sale number "LLDDDDDD-CCCC-DDDDDDD"
                        NSale = new cmdInfo().GetDeviceSerialNumber() + "-" +
                                String.format("%04d", Integer.parseInt(operatorCode)) + "-" + // Pad left with trailing zero
                                String.format("%07d", 1 + Integer.parseInt(fiscalReceipt.getAllreceipt()));  //Next Document number pad left with trailing zero

                        //Open Fiscal bon in current receipt and return number of receipt
                        DocNum = fiscalReceipt.openFiscalReceipt(
                                operatorCode,
                                operatorPassword,
                                NSale,
                                salePoint,
                                false);
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash);


                        //Registration of item for sale
                        cmdReceipt.FiscalReceipt.FiscalSale testSale =
                                new cmdReceipt.FiscalReceipt.FiscalSale(
                                        "Продажба с количество",
                                        "1",
                                        "0.01",
                                        "2.555",
                                        cmdReceipt.FiscalSale.DiscountType.noDiscount,
                                        "",
                                        "",
                                        "kg").add();
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);


                        //VOID of Registered  item for sale (Discount by percentage)
                        testSale.add(
                                "Продажба с количество",
                                "1",
                                "",
                                "-0.01", //Product price, with sign '-' at void operations.
                                "2.555",
                                cmdReceipt.FiscalSale.DiscountType.discountPercentage,
                                "",
                                "kg");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);

                        //SUBTOTAL
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");// No display
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);

                        //Registration of item for sale (Minimum of parameters)
                        testSale.add(
                                "Продажба с минимум параметри",
                                "1",
                                "",
                                "0.01",
                                "",
                                cmdReceipt.FiscalSale.DiscountType.noDiscount,
                                "");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        //SUBTOTAL
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");// No display


                        //Registration of item for sale  (Surcharge by value)
                        testSale.add(
                                "Продажба с надбавка по стойност",
                                "1",
                                "",
                                "0.01",
                                "",
                                cmdReceipt.FiscalSale.DiscountType.surchargeSum,
                                "0.01");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        //SUBTOTAL
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");// No display

                        //Registration of item for sale (Surcharge by percentage )
                        testSale.add(
                                "Продажба процентна надбавка",
                                "1",
                                "",
                                "0.01",
                                "",
                                cmdReceipt.FiscalSale.DiscountType.surchargePercentage,
                                "10");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        //SUBTOTAL
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");// No display

                        //Registration of item for sale (Discount by percentage)
                        testSale.add(
                                "Продажба с отсъпка по стойнст",
                                "1",
                                "",
                                "0.1",
                                "",
                                cmdReceipt.FiscalSale.DiscountType.discountSum,
                                ".09");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        //SUBTOTAL
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");// No display

                        //Registration of item for sale (Discount by percentage)
                        testSale.add(
                                "Продажба с отсъпка в %",
                                "1",
                                "",
                                "0.1",
                                "",
                                cmdReceipt.FiscalSale.DiscountType.discountPercentage,
                                "10");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        //SUBTOTAL
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");// No display

                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        fiscalReceipt.printFreeText("Благодарим Ви!");
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.text);
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");

                        //SUBTOTAL with Discount
                        fiscalReceipt.printSeparatingLine(cmdReceipt.SeparatingLine.dash_space);
                        fiscalReceipt.printFreeText("Отстъпка в междинна сума");
                        testSale.printSubtotal(false, cmdReceipt.FiscalSale.DiscountType.discountSum, "0.13");

                        //
                        String toPay = testSale.readSubtotal(false, cmdReceipt.FiscalSale.DiscountType.noDiscount, "");

                        //TOTAL
                        testSale.saleTotal(
                                cmdReceipt.FiscalSale.PaymentType.cash,
                                toPay
                        );
                        fiscalReceipt.closeFiscalReceipt();
                        DateTime = new cmdConfig.DateTime().getDateTime();
                    } else fiscalReceipt.cancel();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnCancelAll.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    if (fiscalReceipt.isOpen()) {
                        fiscalReceipt.cancel();
                    }
                    if (noFiscalReceipt.isOpen()) {
                        noFiscalReceipt.closeNonFiscalReceipt();
                    }

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnCustomCommand.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    DialogCustomCommand d = new DialogCustomCommand(getActivity());
                    d.show();
                    d.setTitle("Execute Command");
                    d.show();
                    DisplayMetrics displaymetrics = new DisplayMetrics();
                    getActivity().getWindowManager().getDefaultDisplay().getMetrics(displaymetrics);
                    int width = (int) ((int) displaymetrics.widthPixels * 0.9);
                    int height = (int) ((int) displaymetrics.heightPixels * 0.9);
                    d.getWindow().setLayout(width, height);

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });
//43,1[\t]0000[\t]1[\t]0[\t]000729[\t]01-02-19 10:15:45[\t]02636499[\t][\t][\t][\t][\t]
        binder.btnStorno.setOnClickListener(new View.OnClickListener() {

            @Override
            public void onClick(View v) {
                if (DocNum == null) return;
                if (DateTime == null) return;
                if (DocNum < 1) return;
                String operatorCode = "1";// Operator number from 1...30;
                String operatorPassword = "0000"; //Operator password, ascii string of digits. Lenght from 1...8;
                String salePoint = "1"; //Number of point of sale from 1...99999;

                try {
                    cmdReceipt.FiscalReceipt.Storno stornoReceipt = new cmdReceipt.FiscalReceipt.Storno();
                    stornoReceipt.open(operatorCode,
                            operatorPassword,
                            salePoint,
                            cmdReceipt.FiscalReceipt.Storno.StornoType.operator_error,
                            String.valueOf(DocNum),
                            DateTime,
                            new cmdInfo().GetDeviceFMnumber(),
                            false,
                            "",
                            "",
                            "");

                    //Registration of item for sale
                    cmdReceipt.FiscalReceipt.FiscalSale stornoSale = new cmdReceipt.FiscalReceipt.FiscalSale();
                    stornoSale.add(
                            "Сторно:",
                            "1",
                            "",
                            "0.01",
                            "",
                            cmdReceipt.FiscalSale.DiscountType.noDiscount,
                            "");
                    stornoSale.saleTotal(cmdReceipt.PaymentType.cash, "");
                    stornoReceipt.closeFiscalReceipt();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnNonFiscalReceipt.setOnClickListener(new View.OnClickListener() {


            @Override
            public void onClick(View v) {
                try {
                    DocNum = noFiscalReceipt.openNonFiscalReceipt();
                    noFiscalReceipt.printNonFiscalText("Благодарим Ви!");
                    DocNum = noFiscalReceipt.closeNonFiscalReceipt();


                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnPrintConfig.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new PrintConfigFragment(), "print_config_frg");
                    ft.addToBackStack(null);
                    ft.commit();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });

        binder.btnInfoEJ.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new EJ_Fragment(), "ej_frg");
                    ft.addToBackStack(null);
                    ft.commit();

                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }
            }
        });


        binder.btnItems.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                try {
                    FragmentTransaction ft = getActivity().getSupportFragmentManager().beginTransaction();
                    ft.replace(R.id.container, new ItemsFragment(), "items_frg");
                    ft.addToBackStack(null);
                    ft.commit();
                } catch (Exception e) {
                    postToast(e.getMessage());
                    e.printStackTrace();
                }

            }
        });


    }

    private void postToast(final String message) {
        this.getActivity().runOnUiThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getActivity(), message, Toast.LENGTH_LONG).show();
            }
        });
    }

}
